#!/usr/bin/env bash
# SPDX-License-Identifier: MIT
set -euo pipefail

if ! [[ ${#} -eq 1 && -d ${1} &&
	${1} =~ .*/?(ruffle-nightly-(....)-(..)-(..))/? ]]
then
	cat >&2 <<-EOF
		Usage: ${0##*/} ruffle-nightly-YYYY-MM-DD/
		Today's: https://github.com/ruffle-rs/ruffle/archive/refs/tags/nightly-$(date +%F).tar.gz

		This package is messy and does not play that well with pycargoebuild.
		Would also prefer to vendor git crates which is not supported by it.
		So this lets "cargo vendor" handle this instead, will create a vendor
		tarball in the same directory, and echo the "LICENSE" value

		Script is a bit messy but in ebuilds it simply needs to be unpacked
		LICENSE be set, and that's it.

		If this break, feel free to try using pycargoebuild again instead.

		Requires: dev-util/cargo-license, dev-python/pycargoebuild, virtual/rust
	EOF
	exit 1
fi
S=${1}
P=${BASH_REMATCH[1]}
output_tarball=${PWD}/${P}-vendor.tar.xz

T=
trap '
    if [[ ${T} == DEBUG*"${0##*/}".* && -d ${T} ]]; then
        rm -r -- "${T}"
    fi
' EXIT
T=$(mktemp -p '' -d "${0##*/}.XXXXXXXXXX")

# copy sources to keep original pristine
cp -a "${S}" "${T}/${P}"
S=${T}/${P}

echo "--- Vendoring crates..."
pushd -- "${S}" >/dev/null
mkdir -p -- .cargo
cargo vendor --versioned-dirs --locked >> .cargo/config.toml
popd >/dev/null

echo "--- Determining LICENSE for vendored crates..."
LICENSE=
while IFS=$'\t' read -ra license; do
	if [[ ! -v license[1] ]]; then
		case ${license[0]} in
			# add more as needed
			ring) : "ISC AND OpenSSL";;
			nihav_*) : "MIT";;
			*)
				echo "Unknown license for crate: ${license[0]}" >&2
				exit 1
			;;
		esac
		license[1]=${_}
	fi
	LICENSE+=" AND (${license[1]})"
done < <(
	cd -- "${S}" &&
		cargo license -t --avoid-{build,dev}-deps | tail -n +2 | cut -f1,5
)
LICENSE=${LICENSE# AND }

: "$(portageq get_repo_path / gentoo)/metadata/license-mapping.conf"
cp -- "${_}" "${T}/map.conf"
# debatable whether this should be in ::gentoo given it is not SPDX
echo "LicenseRef-UFL-1.0 = UbuntuFontLicense-1.0" >> "${T}/map.conf"

python3 <<-EOF
	from license_expression import get_spdx_licensing
	from pycargoebuild.format import format_license_var
	from pycargoebuild.license import (
        load_license_mapping,
	    spdx_to_ebuild
	)

	license_mapping = open('${T}/map.conf', 'r', encoding='utf-8')
	load_license_mapping(license_mapping)
	license_mapping.close()

	licensing = get_spdx_licensing()
	parsed = licensing.parse('${LICENSE}').simplify()
	pretty = format_license_var(spdx_to_ebuild(parsed), prefix='LICENSE="')

	print('LICENSE+="' + pretty + '" # crates')
EOF

echo "--- Creating '${output_tarball}'..."

cp -p -- "${0}" "${T}/ruffle-vendor.sh" # include a self-copy
chmod 0755 "${T}/ruffle-vendor.sh"

XZ_OPT=-9evT6 $(type -p gtar tar | head -n 1) -C "${T}" \
	-cJf "${output_tarball}" --sort=name --{owner,group}=0 -- \
	"${P}/.cargo/config.toml" "${P}/vendor" ruffle-vendor.sh

echo -n "BLAKE2B: "
b2sum -- "${output_tarball}"
echo -n "SHA512: "
sha512sum -- "${output_tarball}"

echo "--- All done."

# vim: ts=4
